﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Joseph Helm and Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Optimization;
using HeuristicLab.Problems.BinPacking.Interfaces;
using HeuristicLab.Problems.BinPacking.Shapes;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Common;
using HeuristicLab.Parameters;
using HeuristicLab.Encodings.PackingEncoding.PackingPlan;
using HeuristicLab.Data;

namespace HeuristicLab.Problems.BinPacking.Problem {
  [Item("BinPackingProblem", "Abstract class that represents a BinPacking Problem")]
  [StorableClass]
  public abstract class BinPackingProblem<D, B, I> : SingleObjectiveHeuristicOptimizationProblem<IPackingPlanEvaluationAlgorithm, IPackingSolutionCreator> 
    where D : class, IPackingDimensions 
    where B : PackingShape<D>, IPackingBin, new()
    where I : PackingShape<D>, IPackingItem, new () {


    #region Parameter Properties
    public OptionalValueParameter<PackingPlan<D,B,I>> BestKnownSolutionParameter {
      get { return (OptionalValueParameter<PackingPlan<D, B, I>>)Parameters["BestKnownSolution"]; }
    }
    public ValueParameter<ItemList<I>> PackingItemMeasuresParameter {
      get { return (ValueParameter<ItemList<I>>)Parameters["PackingItemMeasures"]; }
    }
    public IFixedValueParameter<IntValue> PackingItemsParameter {
      get { return (IFixedValueParameter<IntValue>)Parameters["PackingItems"]; }
    }
    public IFixedValueParameter<IntValue> LowerBoundParameter {
      get { return (IFixedValueParameter<IntValue>)Parameters["LowerBound"]; }
    }
    public ValueParameter<IPackingPlanEvaluator> PackingPlanEvaluatorParameter {
      get { return (ValueParameter<IPackingPlanEvaluator>)Parameters["PackingPlanEvaluator"]; }
    }
    #endregion

    #region Properties
    public PackingPlan<D, B, I> BestKnownSolution {
      get { return BestKnownSolutionParameter.Value; }
      set { BestKnownSolutionParameter.Value = value; }
    }
    public ItemList<I> PackingItemMeasures {
      get { return PackingItemMeasuresParameter.Value; }
      set { PackingItemMeasuresParameter.Value = value; } 
    }
    public IPackingPlanEvaluator PackingPlanEvaluator {
      get { return PackingPlanEvaluatorParameter.Value; }
      set { PackingPlanEvaluatorParameter.Value = value; }
    }
    #endregion


    [StorableConstructor]
    protected BinPackingProblem(bool deserializing) : base(deserializing) { }
    protected BinPackingProblem(BinPackingProblem<D,B,I> original, Cloner cloner)
      : base(original, cloner) {
    }                 
    protected BinPackingProblem(IPackingPlanEvaluationAlgorithm e, IPackingSolutionCreator c) : base (e, c){
      Parameters.Add(new OptionalValueParameter<PackingPlan<D, B, I>>("BestKnownSolution", "The best known solution of this bin-packing instance."));
      Parameters.Add(new ValueParameter<ItemList<I>>("PackingItemMeasures", "Packing-item data defining the measures of the different items that need to be packed.", new ItemList<I>()));

      Parameters.Add(new FixedValueParameter<IntValue>("PackingItems", "The number of packing-items used in this bin packing instance.", new IntValue()));
      Parameters.Add(new FixedValueParameter<IntValue>("LowerBound", "The lower possible number of bins needed to solve this problem (taken from Dell'Amico, Martello and Vigo; 2002)", new IntValue()));
    }

    protected abstract void InitializeDecoder(); 
    protected abstract void InitializeProblemInstance();
    protected abstract void InitializeOperators();
    protected abstract void InitializeEventHandlers();
  }
}
